from django.forms import Form
from django.test import TestCase
from django.urls import reverse
from django.contrib.auth import get_user_model

User = get_user_model()


class LoginCheckTestCase(TestCase):
    """Basic tests of the login check endpoint"""

    def setUp(self) -> None:
        """Get the url and create a user"""
        self.url = reverse("django_accounts_api:login_check")
        self.user = User.objects.create_user("test")
        return super().setUp()

    def test_get_login_not_authenticated(self):
        """Just hitting this endpoint with no login should return 401"""
        response = self.client.get(self.url)
        assert response.status_code == 401

    def test_get_login_authenticated(self):
        """hitting this endpoint with login should return 200"""
        self.client.force_login(self.user)
        response = self.client.get(self.url)
        assert response.status_code == 200
        assert "name" in response.json()


class LogoutTestCase(TestCase):
    """Basic tests of the logout endpoint"""

    def setUp(self) -> None:
        """Get the url and create a user"""
        self.url = reverse("django_accounts_api:logout")
        self.user = User.objects.create_user("test")
        self.client.force_login(self.user)
        return super().setUp()

    def test_get_logout(self):
        """Just getting this endpoint with no login should return 200"""
        response = self.client.get(self.url)
        assert response.status_code == 200

    def test_post_logout(self):
        """posting to this endpoint with login should return 200"""
        self.client.force_login(self.user)
        response = self.client.post(self.url)
        assert response.status_code == 200


class LoginViewTestCase(TestCase):
    """Basic tests of the partial HTML login view"""

    def setUp(self) -> None:
        """Get the url and create a user"""
        self.url = reverse("django_accounts_api:login")
        self.user = User.objects.create_user("test", password="test")
        return super().setUp()

    def test_get_login(self):
        """An unauthed get should return a form"""
        response = self.client.get(self.url)
        assert response.status_code == 200
        assert "Content-Type" in response.headers
        assert response.headers["Content-Type"].startswith("text/html")
        assert "form" in response.context

    def test_post_login_empty_form(self):
        """A post without username and password should display errors"""
        response = self.client.post(
            self.url, {})
        assert response.status_code == 200
        assert "form" in response.context
        form: Form = response.context["form"]
        assert 'username' in form.errors
        assert 'password' in form.errors
        assert "Content-Type" in response.headers
        assert response.headers["Content-Type"].startswith("text/html")

    def test_post_login_no_password(self):
        """A post without password should display an error"""
        response = self.client.post(
            self.url, dict(
                username='test'
            ))
        assert response.status_code == 200
        assert "form" in response.context
        form: Form = response.context["form"]
        assert 'password' in form.errors
        assert len(form.errors) == 1
        assert "Content-Type" in response.headers
        assert response.headers["Content-Type"].startswith("text/html")

    def test_post_login_incorrect(self):
        """A post with incorrect credentials should display an error"""
        response = self.client.post(
            self.url, dict(
                username='test',
                password="bad"
            ))
        assert response.status_code == 200
        assert "form" in response.context
        form: Form = response.context["form"]
        assert '__all__' in form.errors
        assert len(form.errors) == 1
        assert "Content-Type" in response.headers
        assert response.headers["Content-Type"].startswith("text/html")

    def test_post_login_correct(self):
        """A post with incorrect credentials should return a 201"""
        response = self.client.post(
            self.url, dict(
                username='test',
                password="test"
            ))
        assert response.status_code == 201
        assert "name" in response.json()
        assert "Content-Type" in response.headers
        assert response.headers["Content-Type"].startswith("application/json")


class LoginFlowTestCase(TestCase):
    """A flow test of the login endpoints"""

    def setUp(self) -> None:
        """Get the url and create a user"""
        self.logout_url = reverse("django_accounts_api:logout")
        self.check_url = reverse("django_accounts_api:login_check")
        self.url = reverse("django_accounts_api:login")
        self.user = User.objects.create_user("test", password="test")
        return super().setUp()

    def test_login_logout_flow(self):
        """ user should be unauthed, login, then be authed, then hit log out, then be unauthed"""
        response = self.client.get(self.check_url)
        assert response.status_code == 401
        response = self.client.post(
            self.url, dict(
                username='test',
                password="test"
            ))
        assert response.status_code == 201
        response = self.client.get(self.check_url)
        assert response.status_code == 200
        response = self.client.post(self.logout_url)
        assert response.status_code == 200
        response = self.client.get(self.check_url)
        assert response.status_code == 401



from django.test import TestCase
from django.urls import reverse
from django.contrib.auth import get_user_model

User = get_user_model()


class APIPasswordChangeTestCase(TestCase):
    """Basic tests of the content type json API change password view"""

    def setUp(self) -> None:
        """Get the url and create a user"""
        self.url = reverse("django_accounts_api:password_change")
        self.user = User.objects.create_user("test", password="test")
        return super().setUp()

    def test_json_passwordchange_get_unauthed(self):
        """An unauthed get with accept json should return a 401"""
        response = self.client.get(
            self.url,
            HTTP_ACCEPT='application/json'
        )
        assert response.status_code == 401

    def test_json_passwordchange_get_authed(self):
        """An authed get with accept json should return a 200"""
        self.client.force_login(self.user)
        response = self.client.get(
            self.url,
            HTTP_ACCEPT='application/json'
        )
        assert response.status_code == 200

    def test_json_passwordchange_post_unauthed(self):
        """An unauthed post with accept json should return a 401"""
        response = self.client.post(
            self.url,
            HTTP_ACCEPT='application/json'
        )
        assert response.status_code == 401

    def test_json_passwordchange_post_authed_empty_fields(self):
        """An authed post with accept json should return a 400"""
        self.client.force_login(self.user)
        response = self.client.post(
            self.url,
            HTTP_ACCEPT='application/json'
        )
        assert response.status_code == 400
        response_json = response.json()
        assert len(response_json['errors']) == 3

    def test_json_passwordchange_post_authed_missing_newpass_2(self):
        """An authed post should get a 200"""
        self.client.force_login(self.user)
        response = self.client.post(
            self.url,
            dict(
                old_password="test",
                new_password1="newPawThis",
            ),
            HTTP_ACCEPT='application/json'
        )
        assert response.status_code == 400
        response_json = response.json()
        assert len(response_json['errors']) == 1

    def test_json_passwordchange_post_authed_mismatched_newpass_2(self):
        """An authed post should get a 200"""
        self.client.force_login(self.user)
        response = self.client.post(
            self.url,
            dict(
                old_password="test",
                new_password1="newPawThis",
                new_password2="mismatchedNewPaw",
            ),
            HTTP_ACCEPT='application/json'
        )
        assert response.status_code == 400
        response_json = response.json()
        assert len(response_json['errors']) == 1


class ChangePasswordFlowTestCase(TestCase):
    """A flow test of the login endpoints"""

    def setUp(self) -> None:
        """Get the url and create a user"""
        self.logout_url = reverse("django_accounts_api:logout")
        self.check_url = reverse("django_accounts_api:login_check")
        self.login_url = reverse("django_accounts_api:login")
        self.cp_url = reverse("django_accounts_api:password_change")
        self.user = User.objects.create_user("test", password="test")
        return super().setUp()

    def test_changepasword_flow(self):
        """ user should be unauthed, login, then be authed, then hit log out, then be unauthed"""
        response = self.client.get(self.check_url)
        assert response.status_code == 401
        response = self.client.post(
            self.login_url, dict(
                username='test',
                password="test"
            ))
        assert response.status_code == 201
        response = self.client.post(
            self.cp_url,
            dict(
                old_password="test",
                new_password1="newPawThis",
                new_password2="newPawThis",
            ),
            HTTP_ACCEPT='application/json'
        )
        assert response.status_code == 200
        response = self.client.post(self.logout_url)
        assert response.status_code == 200
        response = self.client.post(
            self.login_url, dict(
                username='test',
                password="newPawThis"
            ))
        assert response.status_code == 201



from http import HTTPStatus
from typing import Any
from django.views.generic import View
from django.conf import settings
from django.contrib.auth.models import User as DjangoUser
from django.contrib.auth import get_user_model
from django.contrib.auth.views import LoginView, LogoutView, PasswordChangeView, PasswordResetView, PasswordResetCompleteView, PasswordResetConfirmView
from django.urls.exceptions import NoReverseMatch
from django.http import HttpRequest, HttpResponse, JsonResponse
from django.urls import reverse
from django.utils.decorators import method_decorator
from django.utils.module_loading import import_string
from django.views.decorators.csrf import csrf_protect, ensure_csrf_cookie
from django.views.decorators.debug import sensitive_post_parameters


class AcceptJsonMixin:
    """
    Adds method to detect if JSON was requested in the Accept header
    """

    def json_response_requested(self: View) -> bool:
        """ does the request want JSON content back?"""
        if "HTTP_ACCEPT" in self.request.META:
            return self.request.META["HTTP_ACCEPT"] == "application/json"
        return False


def manifest(request: HttpRequest) -> HttpResponse:
    """Return a json encoded dictionary {name: path} for views offered
    by Django Accounts API

    :param request: the Django http request
    :type request: HttpRequest
    :return: json encoded name: path dictionary
    :rtype: HttpResponse
    """
    return JsonResponse(dict(
        login=reverse("django_accounts_api:login"),
        logout=reverse("django_accounts_api:logout"),
        password_change=reverse("django_accounts_api:password_change"),
    ))


def _user_details(user: DjangoUser) -> dict:
    """The details of the user to return on success"""
    extra = {}
    add_extra_path = getattr(settings, "ACCOUNT_API_DETAILS", False)
    if add_extra_path:
        try:
            details_func = import_string(add_extra_path)
            extra = details_func(user)
        except ImportError:
            pass

    return dict(
        id=user.pk,
        name=user.get_full_name(),
        **extra
    )


def login_check(request) -> HttpResponse:
    """
    Deprecated (use Login view with Accept application/json)
    200 and details if logged in, 401 if not
    """
    user: DjangoUser = request.user
    if (user.is_authenticated):
        return JsonResponse(_user_details(user))
    else:
        return HttpResponse(status=401)


@method_decorator(ensure_csrf_cookie, name='get')
@method_decorator(sensitive_post_parameters(), name='dispatch')
class Login(AcceptJsonMixin, LoginView):
    '''
    Override the Django login view to be API friendly for json or partial html
    '''
    template_name = "django_accounts_api/login.html"

    def form_valid(self, form):
        """Override redirect behavior to return JSON user details"""
        _repressed_redirect = super().form_valid(form)  # noqa: F841
        return JsonResponse(
            _user_details(self.request.user),
            status=201
        )

    def form_invalid(self, form):
        """Override redirect behavior if json is requested return json errors"""
        if self.json_response_requested():
            return JsonResponse(dict(errors=form.errors), status=400)
        else:
            return super().form_invalid(form)

    def get(self, request: HttpRequest, *args: str, **kwargs: Any) -> HttpResponse:
        """Override the get behavior if json requested return user details"""
        if self.json_response_requested():
            if (request.user.is_authenticated):
                return JsonResponse(_user_details(request.user))
            else:
                return JsonResponse({}, status=HTTPStatus.NO_CONTENT)
        else:
            return super().get(request, *args, **kwargs)


class Logout(LogoutView):
    ''' Override the Django logout view to NOT redirect on successful login
    GET - actually calls POST, but will error in Django 5
    POST - logs out, returns 200
    '''

    def post(self, request, *args, **kwargs):
        _repressed_redirect_or_render = super().post(request, *args, **kwargs)  # noqa: F841
        return HttpResponse(
            status=200
        )


@method_decorator(sensitive_post_parameters(), name='dispatch')
@method_decorator(csrf_protect, name='dispatch')
class PasswordChange(AcceptJsonMixin, PasswordChangeView):
    ''' Override the Django change password view to support API use
    GET - renders a partial change password form - can be accessed without auth
    '''
    template_name = "django_accounts_api/password_change.html"

    def dispatch(self, request: HttpRequest, *args, **kwargs):
        """Django's PasswordChangeView is login required and redirects, we suppress this and 401"""
        if not request.user.is_authenticated:
            return HttpResponse(status=401)
        return super().dispatch(request, *args, **kwargs)

    def get(self, request: HttpRequest, *args: str, **kwargs: Any) -> HttpResponse:
        """Override the get behavior if json requested return user details"""
        if request.user.is_authenticated:
            if (self.json_response_requested()):
                return JsonResponse(_user_details(request.user))
            else:
                return super().post(request, *args, **kwargs)

    def post(self, *args, **kwargs):
        if self.json_response_requested():
            if self.request.user.is_authenticated:
                return super().post(*args, **kwargs)
        else:
            return super().post(*args, **kwargs)

    def form_invalid(self, form):
        """Override redirect behavior if json is requested return json errors"""
        if self.json_response_requested():
            return JsonResponse(dict(errors=form.errors), status=400)
        else:
            return super().form_invalid(form)

    def form_valid(self, form):
        try:
            _repressed_redirect = super().form_valid(form)  # noqa: F841
        except NoReverseMatch:
            pass
        return HttpResponse(status=200)


@method_decorator(sensitive_post_parameters(), name='dispatch')
@method_decorator(csrf_protect, name='dispatch')
class PasswordReset(AcceptJsonMixin, PasswordResetView):
    ''' Override the Django password reset view to support API use
    GET - renders a password reset form - can be accessed without auth
    '''

    template_name = "django_accounts_api/password_reset.html"

    def dispatch(self, request, *args, **kwargs):
        """Override dispatch method to include CSRF and sensitive post parameters protection"""
        return super().dispatch(request, *args, **kwargs)

    def get(self, request, *args, **kwargs):
        """Override the get behavior if json requested return user details"""
        if self.json_response_requested():
            return JsonResponse(_user_details(request.user))
        else:
            return super().get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        """
        Handles POST requests and sends a password reset email if the email is available
        in the DB, or returns "email not found" otherwise.
        """
        email = request.POST.get('email')
        Users = get_user_model()
        user = Users.objects.filter(email=email).first()
        if user:
            form = self.get_form()
            if form.is_valid():
                form.save(
                    subject_template_name='registration/password_reset_subject.txt',
                    email_template_name='registration/password_reset_email.html',
                    use_https=request.is_secure(),
                    from_email=None,
                    request=request,
                )
                if self.json_response_requested():
                    response_data = {
                        "message": "Password reset email has been sent."}
                    return JsonResponse(response_data)
                else:
                    return self.form_valid(form)
            else:
                if self.json_response_requested():
                    response_data = {
                        "message": "Invalid data provided.", "errors": form.errors}
                    return JsonResponse(response_data, status=400)
                else:
                    return self.form_invalid(form)
        else:
            if self.json_response_requested():
                response_data = {"message": "Email not found."}
                return JsonResponse(response_data)
            else:
                form = self.get_form()
                return self.form_invalid(form)

    def form_invalid(self, form):
        """Override redirect behavior if json is requested return json errors"""
        if self.json_response_requested():
            return JsonResponse(dict(errors=form.errors), status=400)
        else:
            return super().form_invalid(form)

    def form_valid(self, form):
        """Override form_valid method to return HttpResponse with status code"""
        try:
            _repressed_redirect = super().form_valid(form)  # noqa: F841
        except NoReverseMatch:
            pass
        return HttpResponse(status=200)


@method_decorator(sensitive_post_parameters(), name='dispatch')
@method_decorator(csrf_protect, name='dispatch')
class PasswordResetConfirm(AcceptJsonMixin, PasswordResetConfirmView):
    """
    Custom PasswordResetConfirmView to support API use
    """
    template_name = 'django_accounts_api/password_reset_confirm.html'

    def dispatch(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse:
        if not self.valid_link(request):
            return HttpResponse(status=400)
        return super().dispatch(request, *args, **kwargs)

    def get(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse:
        if self.json_response_requested():
            return JsonResponse({'message': 'Password reset confirmed'}, status=200)
        return super().get(request, *args, **kwargs)

    def post(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse:
        if self.json_response_requested():
            return JsonResponse({'message': 'Password reset confirmed'}, status=200)
        return super().post(request, *args, **kwargs)

    def form_valid(self, form):
        """
        If the form is valid, redirect to the supplied URL.
        """
        if self.json_response_requested():
            return JsonResponse({'message': 'Password reset confirmed'}, status=200)
        return super().form_valid(form)

    def form_invalid(self, form):
        """
        If the form is invalid, re-render the context data with the
        data-filled form and errors.
        """
        if self.json_response_requested():
            return JsonResponse({'errors': form.errors}, status=400)
        return super().form_invalid(form)


class PasswordResetComplete(PasswordResetCompleteView):
    """
    Custom PasswordResetCompleteView to support API use
    """
    template_name = 'password_reset_complete.html'

    def render_to_response(self, context, **response_kwargs):
        if self.json_response_requested():
            return JsonResponse({'message': 'Password reset successful.'}, status=200)
        else:
            return super().render_to_response(context, **response_kwargs)



from django.test import TestCase
from django.urls import reverse
from django.contrib.auth import get_user_model
from django.utils.encoding import force_bytes
from django.utils.http import urlsafe_base64_encode
from django.contrib.auth.tokens import default_token_generator

User = get_user_model()
test_user_email = "admin@admin.com"


class PasswordResetTestCase(TestCase):
    """Basic tests of the partial HTML password reset view"""

    def setUp(self) -> None:
        """Get the url and create a user"""
        self.user = User.objects.create_user(
            "test", password="test", email=test_user_email
        )
        self.url = reverse(
            "django_accounts_api:password_reset_confirm",
            kwargs=dict(
                uidb64=urlsafe_base64_encode(force_bytes(self.user.pk)),
                token=default_token_generator.make_token(self.user)
            )
        )
        self.invalid_token_url = reverse(
            "django_accounts_api:password_reset_confirm",
            kwargs=dict(
                uidb64=urlsafe_base64_encode(force_bytes(self.user.pk)),
                token="invalid_token"
            )
        )
        # generate the password reset token
        return super().setUp()

    def test_password_reset_confirm_success(self):
        # prepare the data
        data = {
            'new_password1': 'newpass123',
            'new_password2': 'newpass123',
        }

        response = self.client.post(
            self.url, data,
            HTTP_ACCEPT='application/json')
        self.assertEqual(response.status_code, 302)

    def test_password_reset_confirm_failure(self):
        # prepare the data with invalid token
        data = {
            'new_password1': 'newpass123',
            'new_password2': 'newpass123',
        }
        response = self.client.post(
            self.invalid_token_url, data,
            HTTP_ACCEPT='application/json')
        self.assertEqual(response.status_code, 200)
        self.assertContains(
            response, "Password reset unsuccessful")



from django.urls import path
from .views import (
    Login,
    Logout,
    PasswordChange,
    PasswordReset,
    PasswordResetConfirmView,
    PasswordResetComplete,
    login_check,
    manifest
)

app_name = 'django_accounts_api'
urlpatterns = [
    path('', manifest, name='manifest'),
    path('check', login_check, name='login_check'),
    path('login', Login.as_view(), name='login'),
    path('logout', Logout.as_view(), name='logout'),
    path('password_change', PasswordChange.as_view(), name='password_change'),
    path('password_reset', PasswordReset.as_view(), name='password_reset'),
    path(
        "reset/<uidb64>/<token>/",
        PasswordResetConfirmView.as_view(),
        name='password_reset_confirm'
    ),
    path('reset/done/', PasswordResetComplete.as_view(), name='password_reset_complete'),
]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['django_accounts_api', 'django_accounts_api.migrations']

package_data = \
{'': ['*'],
 'django_accounts_api': ['templates/django_accounts_api/*',
                         'templates/registration/*']}

install_requires = \
['django<5.0']

setup_kwargs = {
    'name': 'django-accounts-api',
    'version': '0.3.0a0',
    'description': '',
    'long_description': "# Django Accounts API\n\nScenario to support is a compiled javascript capable frontend needing to provide authentication features over api\n\nCaveat enptor, very early days, still being tested in it's first project\n\n# Requirements\n- Python 3.7 - 3.11\n- Django 3.2 - 4\n\n# Usage\n\n- `pip install ...` or equivalent\n- add `'django_accounts_api',` to INSTALLED_APPS\n- add `path('/accounts_api/', include('django_accounts_api.urls'))` to your urls\n- implement your frontend to use the urls\n\n## Features\n\nSee docs...\n\n\n## Development\n1. Install Poetry https://python-poetry.org/docs/#installation\n\n2. Use a virtual environment https://python-poetry.org/docs/basic-usage/#using-your-virtual-environment\n\n3. `poetry install --with dev --no-root` installs dependencies for development\n\n4. `poetry run pre-commit install` installs the pre-commit hooks\n\n5. `pytest` runs tests\n\n### Tox\n\nTo run tox you will need to make sure that the range of python versions required are available for tox to use.\n\nRecommendation: use pyenv\n- `pyenv install 3.7 3.8 3.9 3.10 3.11`\n- `pyenv local  3.7 3.8 3.9 3.10 3.11`\n- `tox`\n\n### Documentation\n\n- `cd docs`\n- `make html`\n\nTODO: add to tox\n\n### Linting & formatting\n\nTODO: add to tox\n",
    'author': 'PeteCoward',
    'author_email': 'peter@catalpa.io',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from django.test import TestCase, override_settings
from django.urls import reverse
from django.contrib.auth import get_user_model

User = get_user_model()

def extraDetailsUserFunc(user):
    return {
        'hello': 'world'
    }

class APILoginTestCase(TestCase):
    """Basic tests of the content type json API login view"""

    def setUp(self) -> None:
        """Get the url and create a user"""
        self.url = reverse("django_accounts_api:login")
        self.user = User.objects.create_user(
            "test", password="test",
            first_name="Test", last_name="User")
        return super().setUp()

    def test_json_login_get_unauthed(self):
        """An unauthed get with accept json should return a 204"""
        response = self.client.get(
            self.url,
            HTTP_ACCEPT='application/json'
        )
        assert response.status_code == 204

    def test_json_login_get_authed(self):
        """An authed get with accept json should return a 200"""
        self.client.force_login(self.user)
        response = self.client.get(
            self.url,
            HTTP_ACCEPT='application/json'
        )
        json = response.json()
        assert response.status_code == 200
        assert json["id"] == 1
        assert json["name"] == "Test User"

    @override_settings(
        ACCOUNT_API_DETAILS="django_accounts_api.api_login_tests.extraDetailsUserFunc"
    )
    def test_json_login_get_authed_extra(self):
        """An authed get with accept json should return the extra details"""
        self.client.force_login(self.user)
        response = self.client.get(
            self.url,
            HTTP_ACCEPT='application/json'
        )
        assert response.status_code == 200
        json = response.json()
        assert "hello" in json

    def test_json_login_post_no_fields(self):
        """A post accept json without username and password should return 400 and the errors"""
        response = self.client.post(
            self.url, {},
            HTTP_ACCEPT='application/json'
        )
        assert response.status_code == 400
        response_json = response.json()
        assert 'username' in response_json['errors']
        assert 'password' in response_json['errors']

    def test_json_login_post_no_password(self):
        """A post without password should display an error"""
        response = self.client.post(
            self.url, dict(
                username='test'
            ),
            HTTP_ACCEPT='application/json'
        )
        assert response.status_code == 400
        response_json = response.json()
        assert 'password' in response_json['errors']
        assert len(response_json['errors']) == 1

    def test_json_login_post_incorrect(self):
        """A post with incorrect credentials should return an error"""
        response = self.client.post(
            self.url, dict(
                username='test',
                password="bad"
            ),
            HTTP_ACCEPT='application/json'
        )
        assert response.status_code == 400
        response_json = response.json()
        assert '__all__' in response_json['errors']
        assert len(response_json['errors']) == 1

    def test_json_login_post_correct(self):
        """A post with correct credentials should return a 201"""
        response = self.client.post(
            self.url, dict(
                username='test',
                password="test"
            ),
            HTTP_ACCEPT='application/json'
        )
        assert response.status_code == 201
        assert "name" in response.json()



from django.test import TestCase
from django.urls import reverse


class ManifestTestCase(TestCase):
    """Basic tests of the manifest"""

    def setUp(self) -> None:
        """Get the url and create a user"""
        self.url = reverse("django_accounts_api:manifest")
        return super().setUp()

    def test_get_manifest(self):
        """An unauthed get should get a 200 with a form"""
        response = self.client.get(self.url)
        assert response.status_code == 200
        assert "login" in response.json()
        assert "logout" in response.json()
        assert "password_change" in response.json()



from django.test import TestCase
from django.urls import reverse
from django.contrib.auth import get_user_model

User = get_user_model()


class PasswordChangeTestCase(TestCase):
    """Basic tests of the partial HTML password change view"""

    def setUp(self) -> None:
        """Get the url and create a user"""
        self.url = reverse("django_accounts_api:password_change")
        self.user = User.objects.create_user("test", password="test")
        return super().setUp()

    def test_passwordchange_get_unauthed(self):
        """An unauthed get should get a 401"""
        response = self.client.get(self.url)
        assert response.status_code == 401

    def test_passwordchange_post_unauthed(self):
        """An unauthed post should get a 401"""
        response = self.client.post(self.url)
        assert response.status_code == 401

    def test_passwordchange_get_authed(self):
        """An authed get should get a 200 and a form"""
        self.client.force_login(self.user)
        response = self.client.get(self.url)
        assert response.status_code == 200
        assert "form" in response.context

    def test_passwordchange_post_authed(self):
        """An authed post should get a 200 witha form with errors"""
        self.client.force_login(self.user)
        response = self.client.post(self.url)
        assert response.status_code == 200
        assert len(response.context["form"].errors) == 3

    def test_passwordchange_post_authed_ok(self):
        """An authed post should get a 200"""
        self.client.force_login(self.user)
        response = self.client.post(
            self.url,
            dict(
                old_password="test",
                new_password1="newPawThis",
                new_password2="newPawThis"
            )
        )
        assert response.status_code == 200



from django.test import TestCase
from django.urls import reverse
from django.contrib.auth import get_user_model

User = get_user_model()
test_user_email = "admin@admin.com"


class PasswordResetTestCase(TestCase):
    """Basic tests of the partial HTML password reset view"""

    def setUp(self) -> None:
        """Get the url and create a user"""
        self.url = reverse("django_accounts_api:password_reset")
        self.user = User.objects.create_user("test", password="test", email=test_user_email)
        return super().setUp()

    def test_passwordreset_get(self):
        """An unauthed get should get a 200 and a form"""
        response = self.client.get(self.url)
        assert response.status_code == 200
        assert "form" in response.context

    def test_passwordreset_post(self):
        """An post should get a 200 with a form with errors"""
        response = self.client.post(self.url)
        assert response.status_code == 200
        assert len(response.context["form"].errors) == 1

    def test_passwordreset_post_ok(self):
        """An post should get a 200"""
        response = self.client.post(
            self.url,
            dict(
                email=test_user_email
            )
        )
        assert response.status_code == 200
